/*###########################################################################
#                                                                           #
#                       vs1053b header file                                 #
#                                                                           #
###########################################################################*/
#ifndef __VS1053B_H__
#define __VS1053B_H__

#include <avr/io.h>
#include <avr/pgmspace.h>

#define VS_BEEP_VOLUME	180			/* beep volume value					*/
#define VS_FCPU			12288000L	/* VS1053b Operating frequency(Hz)		*/

/* Spi port definition			*/
#ifdef __VS1053B_C__

#define SPI_TYPE 1					/* 0:soft 1:SPIport 2:USART0 3:USART1	*/

#define SPI_MOSI_PIN  PINB			/* MOSI port from PINn					*/
#define SPI_MOSI_BIT  3				/* MOSI bit number						*/

#define SPI_MISO_PIN  PINB			/* MISO port from PINn					*/
#define SPI_MISO_BIT  4				/* MISO bit number						*/

#define SPI_SCK_PIN   PINB			/* SCK port from PINn					*/
#define SPI_SCK_BIT   5				/* SCK bit number						*/

#endif	/* #ifdef __VS1053B_C__	*/

/* DREQ operation macro definition		*/
#define VS_GET_DREQ()		( PIND & _BV(7) )

/* XDCS operation macro definition		*/
#define VS_ENABLE_XDCS()	{ PORTB &= ~_BV(1); }
#define VS_DISABLE_XDCS()	{ PORTB |=  _BV(1); }

/* XCS operation macro definition		*/
#define VS_ENABLE_XCS()		{ PORTB &= ~_BV(2); }
#define VS_DISABLE_XCS()	{ PORTB |=  _BV(2); }

/* XRESET operation macro definition	*/
#define VS_ENABLE_XRESET()	( PORTB &= ~_BV(0) )
#define VS_DISABLE_XRESET()	( PORTB |=  _BV(0) )

/* Register address definition					*/
#define SCI_MODE 		  0x00
#define SCI_STATUS        0x01
#define SCI_BASS		  0x02
#define SCI_CLOCKF		  0x03
#define SCI_DECODE_TIME   0x04
#define SCI_AUDATA		  0x05
#define SCI_WRAM		  0x06
#define SCI_WRAMADDR	  0x07
#define SCI_HDAT0		  0x08
#define SCI_HDAT1		  0x09
#define SCI_AIADDR		  0x0a
#define SCI_VOL			  0x0b
#define SCI_AICTRL0		  0x0c
#define SCI_AICTRL1		  0x0d
#define SCI_AICTRL2		  0x0e
#define SCI_AICTRL3		  0x0f

/* SCI_MODE register value constant definition	*/
#define SM_DIFF 		  0x0001
#define SM_LAYER12  	  0x0002
#define SM_RESET    	  0x0004
#define SM_CANCEL		  0x0008
#define SM_EARSPEAKER_LO  0x0010
#define SM_TESTS		  0x0020
#define SM_STREAM		  0x0040
#define SM_EARSPEAKER_HI  0x0080
#define SM_DACT			  0x0100
#define SM_SDIORD		  0x0200
#define SM_SDISHARE		  0x0400
#define SM_SDINEW		  0x0800
#define SM_ADPCM		  0x1000
#define SM_LINE1		  0x4000
#define SM_CLK_RANGE	  0x8000

/* clock rate constant definition			*/
#define VS_MULTIPLE10	  0x0000				/* VS_FCPU x 1.0			*/
#define VS_MULTIPLE20	  0x2000				/* VS_FCPU x 2.0			*/
#define VS_MULTIPLE25	  0x4000				/* VS_FCPU x 2.5			*/
#define VS_MULTIPLE30	  0x6000				/* VS_FCPU x 3.0			*/
#define VS_MULTIPLE35	  0x7000				/* VS_FCPU x 3.5			*/
#define VS_MULTIPLE40	  0xa000				/* VS_FCPU x 4.0			*/
#define VS_MULTIPLE45	  0xc000				/* VS_FCPU x 4.5			*/
#define VS_MULTIPLE50	  0xe000				/* VS_FCPU x 5.0			*/

/* addition clock rate constant definition	*/
#define VS_ADD_MULTIPLE00 0x0000				/* WMA/AAC playing +0.0		*/
#define VS_ADD_MULTIPLE10 0x0800				/* WMA/AAC playing +1.0		*/
#define VS_ADD_MULTIPLE15 0x1000				/* WMA/AAC playing +1.5		*/
#define VS_ADD_MULTIPLE20 0x1800				/* WMA/AAC playing +2.0		*/

/* functions prototype declaration			*/
void VS1053_initialization( void );
void VS1053_setVolume( uint8_t vol );
void VS1053_beep( uint16_t f ,uint8_t n ,uint8_t length );
void VS1053_write( uint8_t reg ,uint16_t data );
uint16_t VS1053_read( uint8_t reg );
void VS1053_patches150( void );

#endif
